@@ -83,12 +83,12 @@ out123_handle* attribute_align_arg out123_new(void)
 #endif
 
 	out123_clear_module(ao);
-	ao->name = strdup(default_name);
+	ao->name = compat_strdup(default_name);
 	ao->realname = NULL;
 	ao->driver = NULL;
 	ao->device = NULL;
 
-	ao->flags = 0;
+	ao->flags = OUT123_KEEP_PLAYING;
 	ao->rate = -1;
 	ao->gain = -1;
 	ao->channels = -1;
@@ -104,7 +104,7 @@ out123_handle* attribute_align_arg out123_new(void)
 
 void attribute_align_arg out123_del(out123_handle *ao)
 {
-	debug1("out123_del(%p)", (void*)ao);
+	debug2("[%ld]out123_del(%p)", (long)getpid(), (void*)ao);
 	if(!ao) return;
 
 	out123_close(ao); /* TODO: That talks to the buffer if present. */
@@ -134,6 +134,7 @@ static const char *const errstring[OUT123_ERRCOUNT] =
 ,	"basic module system error"
 ,	"unknown parameter code"
 ,	"attempt to set read-only parameter"
+,	"invalid out123 handle"
 };
 
 const char* attribute_align_arg out123_strerror(out123_handle *ao)
@@ -143,7 +144,7 @@ const char* attribute_align_arg out123_strerror(out123_handle *ao)
 
 int out123_errcode(out123_handle *ao)
 {
-	if(!ao) return OUT123_ERR;
+	if(!ao) return OUT123_BAD_HANDLE;
 	else    return ao->errcode;
 }
 
@@ -225,7 +226,7 @@ out123_param( out123_handle *ao, enum out123_parms code
 		case OUT123_NAME:
 			if(ao->name)
 				free(ao->name);
-			ao->name = strdup(svalue ? svalue : default_name);
+			ao->name = compat_strdup(svalue ? svalue : default_name);
 		break;
 		default:
 			ao->errcode = OUT123_BAD_PARAM;
@@ -355,7 +356,7 @@ int read_parameters(out123_handle *ao
 int attribute_align_arg
 out123_open(out123_handle *ao, const char* driver, const char* device)
 {
-	debug3( "out123_open(%p, %s, %s)", (void*)ao
+	debug4( "[%ld]out123_open(%p, %s, %s)", (long)getpid(), (void*)ao
 	,	driver ? driver : "<nil>", device ? device : "<nil>" );
 	if(!ao)
 		return OUT123_ERR;
@@ -388,13 +389,13 @@ out123_open(out123_handle *ao, const char* driver, const char* device)
 
 		/* It is ridiculous how these error messages are larger than the pieces
 		   of memory they are about! */
-		if(device && !(ao->device = strdup(device)))
+		if(device && !(ao->device = compat_strdup(device)))
 		{
 			if(!AOQUIET) error("OOM device name copy");
 			return out123_seterr(ao, OUT123_DOOM);
 		}
 
-		if(!(modnames = strdup(names)))
+		if(!(modnames = compat_strdup(names)))
 		{
 			out123_close(ao); /* Frees ao->device, too. */
 			if(!AOQUIET) error("OOM driver names");
@@ -413,7 +414,7 @@ out123_open(out123_handle *ao, const char* driver, const char* device)
 				if(AOVERBOSE(2))
 					fprintf(stderr, "Chosen output module: %s\n", curname);
 				/* A bit redundant, but useful when it's a fake module. */
-				if(!(ao->driver = strdup(curname)))
+				if(!(ao->driver = compat_strdup(curname)))
 				{
 					out123_close(ao);
 					if(!AOQUIET) error("OOM driver name");
@@ -443,7 +444,7 @@ out123_open(out123_handle *ao, const char* driver, const char* device)
 /* Be resilient, always do cleanup work regardless of state. */
 void attribute_align_arg out123_close(out123_handle *ao)
 {
-	debug1("out123_close(%p)", (void*)ao);
+	debug2("[%ld]out123_close(%p)", (long)getpid(), (void*)ao);
 	if(!ao)
 		return;
 	ao->errcode = 0;
@@ -482,7 +483,7 @@ void attribute_align_arg out123_close(out123_handle *ao)
 int attribute_align_arg 
 out123_start(out123_handle *ao, long rate, int channels, int encoding)
 {
-	debug4( "out123_start(%p, %li, %i, %i)"
+	debug5( "[%ld]out123_start(%p, %li, %i, %i)", (long)getpid()
 	,	(void*)ao, rate, channels, encoding );
 	if(!ao)
 		return OUT123_ERR;
@@ -523,14 +524,16 @@ out123_start(out123_handle *ao, long rate, int channels, int encoding)
 
 void attribute_align_arg out123_pause(out123_handle *ao)
 {
-	debug1("out123_pause(%p)", (void*)ao);
+	debug3( "[%ld]out123_pause(%p) %i", (long)getpid()
+	,	(void*)ao, ao ? (int)ao->state : -1 );
 	if(ao && ao->state == play_live)
 	{
 #ifndef NOXFERMEM
-		if(have_buffer(ao)) buffer_pause(ao);
+		if(have_buffer(ao)){ debug("pause with buffer"); buffer_pause(ao); }
 		else
 #endif
 		{
+debug1("pause without buffer, sensitive=%d", SENSITIVE_OUTPUT(ao));
 			/* Close live devices to avoid underruns. */
 			if( SENSITIVE_OUTPUT(ao)
 			  && ao->close && ao->close(ao) && !AOQUIET )
@@ -542,7 +545,8 @@ void attribute_align_arg out123_pause(out123_handle *ao)
 
 void attribute_align_arg out123_continue(out123_handle *ao)
 {
-	debug1("out123_continue(%p)", (void*)ao);
+	debug3( "[%ld]out123_continue(%p) %i", (long)getpid()
+	,	(void*)ao, ao ? (int)ao->state : -1 );
 	if(ao && ao->state == play_paused)
 	{
 #ifndef NOXFERMEM
@@ -564,7 +568,7 @@ void attribute_align_arg out123_continue(out123_handle *ao)
 
 void attribute_align_arg out123_stop(out123_handle *ao)
 {
-	debug1("out123_stop(%p)", (void*)ao);
+	debug2("[%ld]out123_stop(%p)", (long)getpid(), (void*)ao);
 	if(!ao)
 		return;
 	ao->errcode = 0;
@@ -590,7 +594,8 @@ out123_play(out123_handle *ao, void *bytes, size_t count)
 	size_t sum = 0;
 	int written;
 
-	debug3("out123_play(%p, %p, %"SIZE_P")", (void*)ao, bytes, (size_p)count);
+	debug5( "[%ld]out123_play(%p, %p, %"SIZE_P") (%i)", (long)getpid()
+	,	(void*)ao, bytes, (size_p)count, ao ? (int)ao->state : -1 );
 	if(!ao)
 		return 0;
 	ao->errcode = 0;
@@ -619,6 +624,9 @@ out123_play(out123_handle *ao, void *bytes, size_t count)
 	{
 		errno = 0;
 		written = ao->write(ao, (unsigned char*)bytes, (int)count);
+		debug4( "written: %d errno: %i (%s), keep_on=%d"
+		,	written, errno, strerror(errno)
+		,	ao->flags & OUT123_KEEP_PLAYING );
 		if(written >= 0){ sum+=written; count -= written; }
 		else if(errno != EINTR)
 		{
@@ -629,13 +637,16 @@ out123_play(out123_handle *ao, void *bytes, size_t count)
 			break;
 		}
 	} while(count && ao->flags & OUT123_KEEP_PLAYING);
+
+	debug3( "out123_play(%p, %p, ...) = %"SIZE_P
+	,	(void*)ao, bytes, (size_p)sum );
 	return sum;
 }
 
 /* Drop means to flush it down. Quickly. */
 void attribute_align_arg out123_drop(out123_handle *ao)
 {
-	debug1("out123_drop(%p)", (void*)ao);
+	debug2("[%ld]out123_drop(%p)", (long)getpid(), (void*)ao);
 	if(!ao)
 		return;
 	ao->errcode = 0;
@@ -653,7 +664,7 @@ void attribute_align_arg out123_drop(out123_handle *ao)
 
 void attribute_align_arg out123_drain(out123_handle *ao)
 {
-	debug1("out123_drain(%p)", (void*)ao);
+	debug2("[%ld]out123_drain(%p) ", (long)getpid(), (void*)ao);
 	if(!ao)
 		return;
 	ao->errcode = 0;
@@ -679,7 +690,7 @@ void attribute_align_arg out123_drain(out123_handle *ao)
 
 void attribute_align_arg out123_ndrain(out123_handle *ao, size_t bytes)
 {
-	debug2("out123_ndrain(%p, %"SIZE_P")", (void*)ao, (size_p)bytes);
+	debug3("[%ld]out123_ndrain(%p, %"SIZE_P")", (long)getpid(), (void*)ao, (size_p)bytes);
 	if(!ao)
 		return;
 	ao->errcode = 0;
@@ -840,8 +851,10 @@ static void check_output_module( out123_handle *ao
 		ao->format = -1;
 		result = aoopen(ao);
 		debug1("ao->open() = %i", result);
-		if(result >= 0)
+		if(result >= 0) /* Opening worked, close again. */
 			ao->close(ao);
+		else if(ao->deinit)
+			ao->deinit(ao); /* Failed, ensure that cleanup after init_output() occurs. */
 	}
 	else if(!AOQUIET)
 		error2("Module '%s' init failed: %i", name, result);
@@ -948,7 +961,8 @@ out123_driver_info(out123_handle *ao, char **driver, char **device)
 int attribute_align_arg
 out123_encodings(out123_handle *ao, long rate, int channels)
 {
-	debug3("out123_encodings(%p, %li, %i)", (void*)ao, rate, channels);
+	debug4( "[%ld]out123_encodings(%p, %li, %i)", (long)getpid()
+	,	(void*)ao, rate, channels );
 	if(!ao)
 		return OUT123_ERR;
 	ao->errcode = OUT123_OK;
@@ -998,7 +1012,7 @@ out123_formats( out123_handle *ao, const long *rates, int ratecount
               , int minchannels, int maxchannels
               , struct mpg123_fmt **fmtlist )
 {
-	debug6( "out123_formats(%p, %p, %i, %i, %i, %p)"
+	debug7( "[%ld]out123_formats(%p, %p, %i, %i, %i, %p)", (long)getpid()
 	,	(void*)ao, (void*)rates, ratecount, minchannels, maxchannels
 	,	(void*)fmtlist );
 	if(!ao)
@@ -1080,13 +1094,17 @@ out123_formats( out123_handle *ao, const long *rates, int ratecount
 
 size_t attribute_align_arg out123_buffered(out123_handle *ao)
 {
-	debug1("out123_buffered(%p)", (void*)ao);
+	debug2("[%ld]out123_buffered(%p)", (long)getpid(), (void*)ao);
 	if(!ao)
 		return 0;
 	ao->errcode = 0;
 #ifndef NOXFERMEM
 	if(have_buffer(ao))
-		return buffer_fill(ao);
+	{
+		size_t fill = buffer_fill(ao);
+		debug2("out123_buffered(%p) = %"SIZE_P, (void*)ao, (size_p)fill);
+		return fill;
+	}
 	else
 #endif
 		return 0;
